@@ -10,6 +10,7 @@ from requests.exceptions import HTTPError
 from requests import Response
 from clint.textui import progress
 import six
+import six.moves.urllib as urllib
 
 from . import __version__, session, iarequest, utils
 
@@ -66,7 +67,7 @@ class Item(object):
         :param max_retries: (optional) Maximum number of times to request
                             a website if the connection drops. (default: 1)
 
-        :type archive_session: :class:`ArchiveSession <ArchiveSession>` 
+        :type archive_session: :class:`ArchiveSession <ArchiveSession>`
         :param archive_session: An :class:`ArchiveSession <ArchiveSession>`
                                 object can be provided via the `archive_session`
                                 parameter.
@@ -312,12 +313,28 @@ class Item(object):
         self._json = self.get_metadata()
         return resp
 
+    # s3_is_overloaded()
+    # ____________________________________________________________________________________
+    def s3_is_overloaded(self, access_key=None):
+        u = 'http://s3.us.archive.org'
+        p = dict(
+            check_limit=1,
+            accesskey=access_key,
+            bucket=self.identifier,
+        )
+        r = self.http_session.get(u, params=p)
+        j = r.json()
+        if j.get('over_limit') == 0:
+            return False
+        else:
+            return True
+
     # upload_file()
     # ____________________________________________________________________________________
-    def upload_file(self, body, key=None, metadata={}, headers={},
+    def upload_file(self, body, key=None, metadata=None, headers=None,
                     access_key=None, secret_key=None, queue_derive=True,
                     ignore_preexisting_bucket=False, verbose=False, verify=True,
-                    checksum=False, delete=False, retries=0, retries_sleep=20, 
+                    checksum=False, delete=False, retries=0, retries_sleep=20,
                     debug=False, **kwargs):
         """Upload a single file to an item. The item will be created
         if it does not exist.
@@ -358,7 +375,7 @@ class Item(object):
                         if S3 returns a 503 SlowDown error.
 
         :type retries_sleep: int
-        :param retries_sleep: (optional) Amount of time to sleep between 
+        :param retries_sleep: (optional) Amount of time to sleep between
                               ``retries``.
 
         :type verbose: bool
@@ -377,6 +394,13 @@ class Item(object):
             True
 
         """
+
+        if headers is None:
+            headers = {}
+
+        if metadata is None:
+            metadata = {}
+
         access_key = self.session.access_key if not access_key else access_key
         secret_key = self.session.secret_key if not secret_key else secret_key
 
@@ -462,16 +486,25 @@ class Item(object):
             return _build_request()
         else:
             try:
+                error_msg = ('s3 is overloaded, sleeping for '
+                             '{0} seconds and retrying. '
+                             '{1} retries left.'.format(retries_sleep, retries))
                 while True:
+                    if retries > 0:
+                        if self.s3_is_overloaded(access_key):
+                            time.sleep(retries_sleep)
+                            log.info(error_msg)
+                            if verbose:
+                                sys.stderr.write(' warning: {0}\n'.format(error_msg))
+                            retries -= 1
+                            continue
                     request = _build_request()
                     prepared_request = request.prepare()
                     response = self.http_session.send(prepared_request, stream=True)
                     if (response.status_code == 503) and (retries > 0):
-                        error_msg = ('s3 is overloaded, sleeping for '
-                                     '{0} seconds and retrying. '
-                                     '{1} retries left.'.format(retries_sleep, retries))
                         log.info(error_msg)
-                        sys.stderr.write(' warning: {0}\n'.format(error_msg))
+                        if verbose:
+                            sys.stderr.write(' warning: {0}\n'.format(error_msg))
                         time.sleep(retries_sleep)
                         retries -= 1
                         continue
@@ -536,15 +569,36 @@ class Item(object):
         if not isinstance(files, (list, tuple)):
             files = [files]
 
+        queue_derive = kwargs.get('queue_derive', True)
+
         responses = []
+        file_index = 0
         for f in files:
+            file_index += 1
             if isinstance(f, six.string_types) and os.path.isdir(f):
+                fdir_index = 0
                 for filepath, key in iter_directory(f):
+                    # Set derive header if queue_derive is True,
+                    # and this is the last request being made.
+                    fdir_index += 1
+                    if queue_derive is True and file_index >= len(files) \
+                        and fdir_index >= len(os.listdir(f)):
+                            kwargs['queue_derive'] = True
+                    else:
+                        kwargs['queue_derive'] = False
+
                     if not f.endswith('/'):
                         key = '{0}/{1}'.format(f, key)
                     resp = self.upload_file(filepath, key=key, **kwargs)
                     responses.append(resp)
             else:
+                # Set derive header if queue_derive is True,
+                # and this is the last request being made.
+                if queue_derive is True and file_index >= len(files):
+                    kwargs['queue_derive'] = True
+                else:
+                    kwargs['queue_derive'] = False
+
                 if not isinstance(f, (list, tuple)):
                     key, body = (None, f)
                 else:
@@ -611,7 +665,7 @@ class File(object):
             setattr(self, key, _file[key])
         base_url = '{protocol}//archive.org/download/{identifier}'.format(**item.__dict__)
         self.url = '{base_url}/{name}'.format(base_url=base_url,
-                                              name=name.encode('utf-8'))
+                                              name=urllib.parse.quote(name.encode('utf-8')))
 
     # __repr__()
     # ____________________________________________________________________________________
@@ -677,7 +731,7 @@ class File(object):
         request = iarequest.S3Request(
             method='DELETE',
             url=url,
-            headers={'cascade_delete': cascade_delete},
+            headers={'x-archive-cascade-delete': int(cascade_delete)},
             access_key=access_key,
             secret_key=secret_key
         )
@@ -685,6 +739,11 @@ class File(object):
             return request
         else:
             if verbose:
-                sys.stdout.write(' deleting file: {0}\n'.format(self.name.encode('utf-8')))
+                msg = ' deleting: {0}'.format(self.name.encode('utf-8'))
+                if cascade_delete:
+                    msg += ' and all derivative files.\n'
+                else:
+                    msg += '\n'
+                sys.stdout.write(msg)
             prepared_request = request.prepare()
             return self._item.http_session.send(prepared_request)
